//global variables
var gUrlnav = null;

/*********************************************************************************
 *  function:     urlnavLoad
 *	description:  function called on windows load
 ********************************************************************************/
function urlnavLoad() 
{
  //setup global variables
  gUrlnav = new urlnav();
  
  //add progress listener
  getBrowser().addProgressListener(gUrlnav, Components.interfaces.nsIWebProgress.NOTIFY_ALL);   
}

/*********************************************************************************
 *  function:     urlnavUnload
 *	description:  function called on windows unload
 ********************************************************************************/
function urlnavUnload() 
{
  //remove progress listener
  try {
      getBrowser().removeProgressListener(gUrlnav);
  } catch(e) {}
  
  //destroy global variables
  gUrlnav.destroy();
  gUrlnav = null;
}

/*********************************************************************************
 *  function:     urlnav
 *	description:  global object
 ********************************************************************************/
function urlnav() 
{
  this.init();
}

urlnav.prototype = {

  domain: "urlnav.",
  branch: null,
  prefs: null,
  els: null,
  slideshow: false,
  slideTimer: null,
  slideDir: null,
  
  QueryInterface : function(aIID)
  {
    if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
        aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
        aIID.equals(Components.interfaces.nsIXULBrowserWindow) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },
    
  init: function()
  {
    //setup preference service
    var prefService = "@mozilla.org/preferences-service;1";
    prefService = Components.classes[prefService].getService(Components.interfaces.nsIPrefService); 
    this.branch = prefService.getBranch(null);

    //get preferences
    this.prefs = new Object();
    this.prefs["urlnav.increment"] = getPref("urlnav.increment", "int", this.branch, 1); 
    this.prefs["urlnav.firsttime"] = getPref("urlnav.firsttime", "bool", this.branch, true); 
    this.prefs["urlnav.history.updown"] = getPref("urlnav.history.updown", "bool", this.branch, false);
    this.prefs["urlnav.history.back"] = getPref("urlnav.history.back", "bool", this.branch, true);     
    this.prefs["urlnav.buttons.hide"] = getPref("urlnav.buttons.hide", "bool", this.branch, true); 
    this.prefs["urlnav.buttons.dir"] = getPref("urlnav.buttons.dir", "string", this.branch, "ltr"); 
    this.prefs["urlnav.slideshow.dir"] = getPref("urlnav.slideshow.dir", "string", this.branch, "up"); 
    this.prefs["urlnav.slideshow.delay"] = getPref("urlnav.slideshow.delay", "int", this.branch, 3);   
    
    //get command elements
    this.els = new Object();
    this.els["capture"] = document.getElementById("bcrCapture");
    this.els["release"] = document.getElementById("bcrRelease");
    this.els["up"] = document.getElementById("bcrUp");
    this.els["down"] = document.getElementById("bcrDown");        
    this.els["back"] = document.getElementById("bcrBack");
    this.els["slideshow"] = document.getElementById("bcrSlideshow");
                    
    //set display based on hide and direction
    this.update();

    //do first time prompt
    if (this.prefs["urlnav.firsttime"])
      window.setTimeout("gUrlnav.firstTime()", 50);
    
    //add preference observer
    var pbi = this.branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    pbi.addObserver(this.domain, this, false); 
  },
  
  destroy: function()
  {
    try {
      //remove preference observer
      var pbi = this.branch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
      pbi.removeObserver(this.domain, this);  
    } catch(e) {}
    
    //end slideshow timer
    if (this.slideTimer)
      window.clearTimeout(this.slideTimer);
      
    //destroy variables
    this.prefs = null;
    this.domain = null;
    this.branch = null;
    this.slideshow = null;
    this.slideTimer = null;
    this.slideDir = null;
    this.els = null;   
  },
  
  command: function(aCommand, aEvent, aURI)
  {
    //get current browser
    var browser = getBrowser().mCurrentBrowser;
    var uri;
    
    //do command      
    switch (aCommand) {
      case "capture":
        //create uri list
        uri = (!aURI) ? getBrowser().currentURI.spec : aURI; 
        browser.urlnavList = new urlnavList(uri, this.prefs["urlnav.increment"]);      
        break;
      case "up":
        browser.urlnavList.move("up", this.prefs["urlnav.increment"]);      
        if (this.prefs["urlnav.history.updown"])
          this.openHistory(browser.urlnavList.current);
        else    
          getBrowser().loadURI(browser.urlnavList.current);
        break;
      case "down":
        browser.urlnavList.move("down", this.prefs["urlnav.increment"]);
        if (this.prefs["urlnav.history.updown"])
          this.openHistory(browser.urlnavList.current);
        else    
          getBrowser().loadURI(browser.urlnavList.current);
        break;
      case "back":     
        browser.urlnavList.move("back", this.prefs["urlnav.increment"]);
        if (this.prefs["urlnav.history.updown"])
          this.openHistory(browser.urlnavList.current);
        else    
          getBrowser().loadURI(browser.urlnavList.current);
        break;
      case "slideshow":
        if (this.slideshow) {
          this.slideshow = false;
          this.slideDir = null;
          if (this.slideTimer)
            window.clearTimeout(this.slideTimer);
        }
        else {
          this.slideshow = true;
          if (!aEvent)
            this.slideDir = this.prefs["urlnav.slideshow.dir"];
          else {
            if (aEvent.ctrlKey) {
              if (this.prefs["urlnav.slideshow.dir"] == "up")
                this.slideDir = "down";
              else
                this.slideDir = "up";
            }
            else
              this.slideDir = this.prefs["urlnav.slideshow.dir"];
            
          }
          browser.urlnavList.move(this.slideDir, this.prefs["urlnav.increment"]);
          getBrowser().loadURI(browser.urlnavList.current);  
        }
        break;
      case "release":
      default:
        browser.urlnavList = null;
        break;
    }
    
    //update display      
    this.update();
  },
  
  openHistory: function(aURI)
  {
    var sessionHistory = getWebNavigation().sessionHistory;
    var i;
    var entry;
    
    for (i = sessionHistory.count -1; i >= 0; --i) {
      entry = sessionHistory.getEntryAtIndex(i, false);
      if (entry.URI.spec == aURI) {
        getWebNavigation().gotoIndex(i);
        return;
      }    
    }
    getBrowser().loadURI(aURI);
    return;
  },
  
  firstTime: function()
  {
    //setup prompt service
    var str = document.getElementById("urlnavBundle");
    var brandBundle = document.getElementById("bundle_brand");
  
    var title = str.getString("firsttime.title");
    var brandName = brandBundle.getString("brandFullName");
    var message = str.getFormattedString("firsttime.message",[brandName]);
    var checkbox = str.getString("firsttime.checkbox");
    var showAgain = {value: false};
    
    //prompt for customize  
    const IPS = Components.interfaces.nsIPromptService;
    var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(IPS);
    var rv = ps.confirmEx(window, title, message, 
                       (IPS.BUTTON_TITLE_YES * IPS.BUTTON_POS_0) + 
                       (IPS.BUTTON_TITLE_NO * IPS.BUTTON_POS_1),
                       null, null, null, checkbox, showAgain);
    
    // customize if yes                    
    if (rv == 0)
      BrowserCustomizeToolbar(); 
    
    //set first time pref  
    setPref("urlnav.firsttime", "bool", this.branch, showAgain.value);  
  },
  
  update: function()
  {
    var str = document.getElementById("urlnavBundle");
    var browser = getBrowser().mCurrentBrowser;
    var i;
    
    //set button direction
    if (document.getElementById("itemUrlnav"))
      document.getElementById("itemUrlnav").setAttribute("dir", this.prefs["urlnav.buttons.dir"]);
    
    //set disabled state, tooltip, and collapsed
    if (typeof(browser.urlnavList) == "object" && browser.urlnavList != null) {
      for (i in this.els) {
        if (i != "capture") { 
          this.els[i].removeAttribute("disabled");
          this.els[i].removeAttribute("collapsed");
        }
      }        
      this.els["back"].setAttribute("tooltiptext", str.getFormattedString("back.enabled", [browser.urlnavList.start]));    
      this.els["down"].setAttribute("tooltiptext", str.getFormattedString("down.enabled", [browser.urlnavList.down]));
      this.els["up"].setAttribute("tooltiptext", str.getFormattedString("up.enabled", [browser.urlnavList.up]));            
    }
    else {
      this.slideshow = false;
      this.slideDir = null;
      browser.urlnavList = null;
      if (this.slideTimer)
        window.clearTimeout(this.slideTimer);
      for (i in this.els) {
        if (i != "capture") { 
          this.els[i].setAttribute("disabled","true");
          if (this.prefs["urlnav.buttons.hide"])
            this.els[i].setAttribute("collapsed", "true");
          else
            this.els[i].removeAttribute("collapsed");
        }
      }
      this.els["back"].setAttribute("tooltiptext", str.getString("back.disabled"));    
      this.els["down"].setAttribute("tooltiptext", str.getString("down.disabled"));
      this.els["up"].setAttribute("tooltiptext", str.getString("up.disabled"));                                    
    }
    
    //set slideshow running
    if (this.slideshow) {
      this.els["slideshow"].setAttribute("checked", "true");
      this.els["slideshow"].setAttribute("tooltiptext", str.getString("slideshow.enabled"));
    }
    else {
      this.els["slideshow"].removeAttribute("checked");
      this.els["slideshow"].setAttribute("tooltiptext", str.getString("slideshow.disabled"));      
    }
  },

  observe: function(aSubject, aTopic, aPrefName)
  {
    //get changed preference
    switch (this.branch.getPrefType(aPrefName)) {
      case 128: 
        this.prefs[aPrefName] = getPref(aPrefName, "bool", this.branch);
        break;
      case 64:
        this.prefs[aPrefName] = getPref(aPrefName, "int", this.branch);
        break;
      case 32:
      default:
        this.prefs[aPrefName] = getPref(aPrefName, "string", this.branch);
        break;        
    }
    
    //update display    
    this.update();
  },

  onLocationChange: function(aWebProgress, aRequest, aLocation) 
  {
    if (aWebProgress.DOMWindow == content)
      //update display
      this.update();
  },

  onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) 
  {
    const nsIWebProgressListener = Components.interfaces.nsIWebProgressListener;
    const nsIChannel = Components.interfaces.nsIChannel;
    
    if (aStateFlags & nsIWebProgressListener.STATE_STOP) {
      if (aRequest) {
        if (aWebProgress.DOMWindow == content) {
          //update display
          this.update();
          
          //do slideshow
          if (this.slideshow)
            this.slideTimer = window.setTimeout("gUrlnav.command(gUrlnav.slideDir, null, null)",this.prefs["urlnav.slideshow.delay"]*1000);
        }
      }
    }
  },
          
  onDragOver: function(aEvent, aFlavour, aDragSession)
  {
    aEvent.target.setAttribute("dragover", "true");
    return true;
  },
  
  onDragExit: function (aEvent, aDragSession)
  {
    aEvent.target.removeAttribute("dragover");
  },
  
  onDrop: function (aEvent, aXferData, aDragSession)
  {
    var xferData = aXferData.data.split("\n");
    var uri = xferData[0] ? xferData[0] : xferData[1];
    if (uri)
      this.command("capture", null, uri);
  },
  
  getSupportedFlavours: function ()
  {
    var flavourSet = new FlavourSet();
    flavourSet.appendFlavour("application/x-moz-file", "nsIFile");
    flavourSet.appendFlavour("text/x-moz-url");
    flavourSet.appendFlavour("text/unicode");
    return flavourSet;
  },
  
  //needed for progress listener  
  onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) {},
  onLinkIconAvailable: function(aHref) {},      
  onProgressChange: function (aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) {},
  onSecurityChange: function(aWebProgress, aRequest, aState) {}
}

/*********************************************************************************
 *  function:     urlnavList
 *	description:  object used to hold urls
 ********************************************************************************/
function urlnavList(aURI, aIncrement)
{
  this.start = aURI;
  this.current = this.start;
  this.up = this.increment(this.current, aIncrement);
  this.down = this.increment(this.current, -1*aIncrement);
}

urlnavList.prototype = {
  move: function(aDirection, aIncrement)
  {
    switch(aDirection) {
      case "back":
        this.current = this.start;
        this.up = this.increment(this.current, aIncrement);
        this.down = this.increment(this.current, -1*aIncrement);
        break;
      case "down":
        this.up = this.current;
        this.current = this.increment(this.current, -1*aIncrement);
        this.down = this.increment(this.current, -1*aIncrement);
        break;
      case "up":
      default:               
        this.down = this.current;
        this.current = this.increment(this.current, aIncrement);
        this.up = this.increment(this.current, aIncrement);
        break;
    } 
  },
  
  increment: function(aString, aIncrement)
  {
    var endIndex, startIndex, len, oldNum, newNum;
    len = aString.length;
    
    for (endIndex = len-1; endIndex >=0; --endIndex)
      if (this.isDigit(aString.charAt(endIndex))) 
      {
        for(startIndex = endIndex-1; startIndex >=0; --startIndex)
          if (!this.isDigit(aString.charAt(startIndex)))
            break;
        break;
      }
    
    ++startIndex;
    
    if (endIndex < 0)
      return aString;
    
    oldNum = aString.substring(startIndex, endIndex+1);
    newNum = '' + (parseInt(oldNum,10) + aIncrement); 
    while (newNum.length < oldNum.length)
      newNum = '0' + newNum;
       
    return aString.substring(0,startIndex) + newNum + aString.slice(endIndex+1);  
  },
  
  isDigit: function(c)
  {
    var c_num = parseInt(c);
    return (0 <= c_num && c_num <= 9);  
  }
}

/*********************************************************************************
 *  function:     getPref
 *	description:  function used to return pref value
 ********************************************************************************/
function getPref(aString, aType, aBranch, aDefault)
{
  try {
    var val;
    switch(aType) {
      case "bool":
        val = aBranch.getBoolPref(aString);
        break;
      case "int":
        val = aBranch.getIntPref(aString);
        break;
      case "color":
      case "string":
      default:
        val = aBranch.getCharPref(aString);
        break;  
    }
    return val;
  } 
  catch(e) {
    if (aDefault) {
      setPref(aString, aType, aDefault);
      return aDefault;
    }
  }
}  

/*********************************************************************************
 *  function:     setPref
 *	description:  function used to set pref value
 ********************************************************************************/
function setPref(aString, aType, aBranch, aValue)
{
  try {
    switch(aType) {
      case "bool":
        aBranch.setBoolPref(aString, aValue);
        break;
      case "int":
        aBranch.setIntPref(aString, aValue);
        break;
      case "color":
      case "string":
      default:
        aBranch.setCharPref(aString, aValue);
        break;    
    }
  } catch(e) {}
}